@@ -15,6 +15,7 @@ typedef struct {
 
     PyObject *dict;
     PyObject *wrapped;
+    PyObject *weakreflist;
 } WraptObjectProxyObject;
 
 PyTypeObject WraptObjectProxy_Type;
@@ -48,6 +49,7 @@ static PyObject *WraptObjectProxy_new(PyTypeObject *type,
 
     self->dict = PyDict_New();
     self->wrapped = NULL;
+    self->weakreflist = NULL;
 
     return (PyObject *)self;
 }
@@ -153,6 +155,9 @@ static void WraptObjectProxy_dealloc(WraptObjectProxyObject *self)
 {
     PyObject_GC_UnTrack(self);
 
+    if (self->weakreflist != NULL)
+        PyObject_ClearWeakRefs((PyObject *)self);
+
     WraptObjectProxy_clear(self);
 
     Py_TYPE(self)->tp_free(self);
@@ -206,6 +211,16 @@ static PyObject *WraptObjectProxy_str(WraptObjectProxyObject *self)
 
 static PyObject *WraptObjectProxy_add(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -219,6 +234,16 @@ static PyObject *WraptObjectProxy_add(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_subtract(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -233,6 +258,16 @@ static PyObject *WraptObjectProxy_subtract(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_multiply(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -247,6 +282,16 @@ static PyObject *WraptObjectProxy_multiply(PyObject *o1, PyObject *o2)
 #if PY_MAJOR_VERSION < 3
 static PyObject *WraptObjectProxy_divide(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -261,6 +306,16 @@ static PyObject *WraptObjectProxy_divide(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_remainder(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -274,6 +329,16 @@ static PyObject *WraptObjectProxy_remainder(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_divmod(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -288,6 +353,16 @@ static PyObject *WraptObjectProxy_divmod(PyObject *o1, PyObject *o2)
 static PyObject *WraptObjectProxy_power(PyObject *o1, PyObject *o2,
         PyObject *modulo)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -361,6 +436,16 @@ static PyObject *WraptObjectProxy_invert(WraptObjectProxyObject *self)
 
 static PyObject *WraptObjectProxy_lshift(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -374,6 +459,16 @@ static PyObject *WraptObjectProxy_lshift(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_rshift(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -387,6 +482,16 @@ static PyObject *WraptObjectProxy_rshift(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_and(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -400,6 +505,16 @@ static PyObject *WraptObjectProxy_and(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_xor(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -413,6 +528,16 @@ static PyObject *WraptObjectProxy_xor(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_or(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -806,6 +931,16 @@ static PyObject *WraptObjectProxy_inplace_or(WraptObjectProxyObject *self,
 
 static PyObject *WraptObjectProxy_floor_divide(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -819,6 +954,16 @@ static PyObject *WraptObjectProxy_floor_divide(PyObject *o1, PyObject *o2)
 
 static PyObject *WraptObjectProxy_true_divide(PyObject *o1, PyObject *o2)
 {
+    if (!((WraptObjectProxyObject *)o1)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!((WraptObjectProxyObject *)o2)->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
     if (PyObject_IsInstance(o1, (PyObject *)&WraptObjectProxy_Type))
         o1 = ((WraptObjectProxyObject *)o1)->wrapped;
 
@@ -1247,18 +1392,13 @@ static PyObject *WraptObjectProxy_get_wrapped(
 static int WraptObjectProxy_set_wrapped(WraptObjectProxyObject *self,
         PyObject *value)
 {
-    if (!self->wrapped) {
-      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
-      return -1;
-    }
-
     if (!value) {
         PyErr_SetString(PyExc_TypeError, "__wrapped__ must be an object");
         return -1;
     }
 
     Py_INCREF(value);
-    Py_DECREF(self->wrapped);
+    Py_XDECREF(self->wrapped);
 
     self->wrapped = value;
 
@@ -1372,9 +1512,6 @@ static int WraptObjectProxy_setattro(
 #endif
     }
 
-    if (PyObject_RichCompareBool(name, wrapped_str, Py_EQ) == 1)
-        return PyObject_GenericSetAttr((PyObject *)self, name, value);
-
     if (PyObject_HasAttr((PyObject *)Py_TYPE(self), name))
         return PyObject_GenericSetAttr((PyObject *)self, name, value);
 
@@ -1551,7 +1688,7 @@ PyTypeObject WraptObjectProxy_Type = {
     (traverseproc)WraptObjectProxy_traverse, /*tp_traverse*/
     (inquiry)WraptObjectProxy_clear, /*tp_clear*/
     (richcmpfunc)WraptObjectProxy_richcompare, /*tp_richcompare*/
-    0,                      /*tp_weaklistoffset*/
+    offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     (getiterfunc)WraptObjectProxy_iter, /*tp_iter*/
     0,                      /*tp_iternext*/
     WraptObjectProxy_methods, /*tp_methods*/
@@ -1622,7 +1759,7 @@ PyTypeObject WraptCallableObjectProxy_Type = {
     0,                      /*tp_traverse*/
     0,                      /*tp_clear*/
     0,                      /*tp_richcompare*/
-    0,                      /*tp_weaklistoffset*/
+    offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     0,                      /*tp_iter*/
     0,                      /*tp_iternext*/
     0,                      /*tp_methods*/
@@ -2117,7 +2254,7 @@ PyTypeObject WraptFunctionWrapperBase_Type = {
     (traverseproc)WraptFunctionWrapperBase_traverse, /*tp_traverse*/
     (inquiry)WraptFunctionWrapperBase_clear, /*tp_clear*/
     0,                      /*tp_richcompare*/
-    0,                      /*tp_weaklistoffset*/
+    offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     0,                      /*tp_iter*/
     0,                      /*tp_iternext*/
     0,                      /*tp_methods*/
@@ -2350,7 +2487,7 @@ PyTypeObject WraptBoundFunctionWrapper_Type = {
     0,                      /*tp_traverse*/
     0,                      /*tp_clear*/
     0,                      /*tp_richcompare*/
-    0,                      /*tp_weaklistoffset*/
+    offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     0,                      /*tp_iter*/
     0,                      /*tp_iternext*/
     0,                      /*tp_methods*/
@@ -2490,7 +2627,7 @@ PyTypeObject WraptFunctionWrapper_Type = {
     0,                      /*tp_traverse*/
     0,                      /*tp_clear*/
     0,                      /*tp_richcompare*/
-    0,                      /*tp_weaklistoffset*/
+    offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     0,                      /*tp_iter*/
     0,                      /*tp_iternext*/
     0,                      /*tp_methods*/
